/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            ImageTexture.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef IMAGETEXTURE_H
#define IMAGETEXTURE_H

#include "X3DTexture2DNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/texturing.html">
/// Texturing component</a> defining an ImageTexture
/// (<a href="../../doc/X3DSpecs/Part01/components/texturing.html#ImageTexture">
/// documentation</a>).

class ImageTexture : public X3DTexture2DNode
{
public:  
  /// Constructor.
  ImageTexture();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~ImageTexture();
  
  /// Sets the url attribute.
  void setUrl(const MFString &url);

  /// Gets the url attribute.
  inline const MFString &getUrl() const {return _url;};
    
  /// Loads the attributes from a FileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:  
  /// Copy constructor.	
  ImageTexture(const ImageTexture &N);

private:  
  /// url.
  MFString _url;  
};

}
}

#endif
