/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            LineProperties.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef LINEPROPERTIES_H
#define LINEPROPERTIES_H

#include "X3DTypes.h"
#include "X3DAppearanceChildNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining a LineProperties
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#LineProperties">
/// documentation</a>).

class LineProperties : public X3DAppearanceChildNode
{
public:
  /// Constructor.
  LineProperties();
  /// Constructs a Material from its attributes.
  LineProperties(SFInt32 lineStyle, 
                 SFFloat width);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor. 
  virtual ~LineProperties();
  
  /// Sets the LineStyle attribute.
  void setLineStyle(SFInt32 lineStyle);
  /// Sets the Width attribute.
  void setWidth(SFFloat width);
  
  /// Gets the LineStyle attribute.
  inline SFInt32 getLineStyle() const {return _lineStyle;};
  /// Gets the Width attribute.
  inline SFFloat getWidth() const {return _width;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:  
  /// Copy constructor.		 
  LineProperties(const LineProperties &L);

private:    
  /// Line style.
  SFInt32 _lineStyle;
  /// Width.
  SFFloat _width;
};

}
}

#endif
