/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            NurbsPositionInterpolator.h                     //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef NURBSPOSITIONINTERPOLATOR
#define NURBSPOSITIONINTERPOLATOR

#include "X3DTypes.h"
#include "X3DInterpolatorNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/NURBS.html">
/// NURBS component</a> defining an NurbsPositionInterpolator
/// (<a href="../../doc/X3DSpecs/Part01/components/NURBS.html#NurbsPositionInterpolator">
/// documentation</a>).

class NurbsPositionInterpolator : public X3DInterpolatorNode
{
public:
  /// Constructor.
  NurbsPositionInterpolator();
  /// Constructs a NurbsPositionInterpolator from its attributes.
  NurbsPositionInterpolator(SFBool         fractionAbsolute,
                            SFInt32        dimension,
                            const MFFloat  &key,
                            const MFVec3f  &keyValue,
                            const MFDouble &keyWeight,
                            const MFDouble &knot,
                            SFInt32        order);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~NurbsPositionInterpolator();

  /// Sets the fractionAbsolute attribute.
  void setFractionAbsolute(SFBool fractionAbsolute);
  /// Sets the dimension attribute.
  void setDimension(SFInt32 dimension);
  /// Sets the keyValue attribute.
  void setKeyValue(const MFVec3f &keyValue);
  /// Sets the keyWeight attribute.
  void setKeyWeight(const MFDouble &keyWeight);
  /// Sets the knot attribute.
  void setKnot(const MFDouble &knot);
  /// Sets the order attribute.
  void setOrder(SFInt32 order);

  /// Gets the fractionAbsolute attribute.
  inline SFBool getFractionAbsolute() const {return _fractionAbsolute;};
  /// Gets the dimension attribute.
  inline SFInt32 getDimenstion() const {return _dimension;};
  /// Gets the keyValue attribute.
  inline const MFVec3f &getKeyValue() const {return _keyValue;};
  /// Gets the keyWeight attribute.
  inline const MFDouble &getKeyWeight() const {return _keyWeight;};
  /// Gets the knot attribute.
  inline const MFDouble &getKnot() const {return _knot;};
  /// Gets the order attribute.
  inline SFInt32 getOrder() const {return _order;};

  /// Loads the attributes from a X3DFileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  NurbsPositionInterpolator(const NurbsPositionInterpolator &N);

private:
  /// ???
  SFBool _fractionAbsolute;
  /// Number of control points.
  SFInt32 _dimension;
  /// Values of the piecewise function corresponding to the key attribute.
  MFVec3f _keyValue;
  /// Weight values assigned to each control point.
  MFDouble _keyWeight;
  /// Knot vector.
  MFDouble _knot;
  /// Order of the curve.
  SFInt32 _order;
};

}
}

#endif
