/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            VisitingFunctions.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef VISITINGFUNCTIONS_H
#define VISITINGFUNCTIONS_H

#include "EnterFunction.h"
#include "WalkOnFunction.h"
#include "LeaveFunction.h"

namespace X3DTK {

/// Concrete class defining the concept of visiting functions, by grouping the 3 
/// visiting functions: enter, walkOn, leave.

class VisitingFunctions
{
public:
  /// Constructor.
  VisitingFunctions(const EnterFunction *enter, const WalkOnFunction *walkon, const LeaveFunction *leave);
  /// Destructor.
  ~VisitingFunctions();
  
  /// Sets the enter function.
  void setEnterFunction(const EnterFunction *enter);
  /// Sets the walkOn function.
  void setWalkOnFunction(const WalkOnFunction *walkon);
  /// Sets the leave function.
  void setLeaveFunction(const LeaveFunction *leave);

  /// Enter the node.
  inline void enter(SFAbstractNode N) const {enter_->enter(N);};
  /// continue the node.
  inline bool walkOn(SFAbstractNode N, SFAbstractNode child = 0) const {return walkon_->walkOn(N, child);};  
  /// leave the node.
  inline void leave(SFAbstractNode N) const  {leave_->leave(N);};

private:
  EnterFunction *enter_;
  WalkOnFunction *walkon_;
  LeaveFunction *leave_;
};

}

#endif
