#include "DefaultNURBSCreator.h"
#include <Contour2D.h>
#include <ContourPolyline2D.h>
#include <NurbsCurve.h>
#include <NurbsCurve2D.h>
#include <NurbsGroup.h>
#include <NurbsPositionInterpolator.h>
#include <NurbsSurface.h>
#include <NurbsTextureSurface.h>
#include <TrimmedSurface.h>

namespace X3DTK {
namespace X3D {

DefaultNURBSCreator::DefaultNURBSCreator()
{
  defineNewCreationFunction<DefaultNURBSCreator>("Contour2D",                 &DefaultNURBSCreator::createContour2D);
  defineNewCreationFunction<DefaultNURBSCreator>("ContourPolyline2D",         &DefaultNURBSCreator::createContourPolyline2D);
  defineNewCreationFunction<DefaultNURBSCreator>("NurbsCurve",                &DefaultNURBSCreator::createNurbsCurve);
  defineNewCreationFunction<DefaultNURBSCreator>("NurbsCurve2D",              &DefaultNURBSCreator::createNurbsCurve2D);
  defineNewCreationFunction<DefaultNURBSCreator>("NurbsGroup",                &DefaultNURBSCreator::createNurbsGroup);
  defineNewCreationFunction<DefaultNURBSCreator>("NurbsPositionInterpolator", &DefaultNURBSCreator::createNurbsPositionInterpolator);
  defineNewCreationFunction<DefaultNURBSCreator>("NurbsSurface",              &DefaultNURBSCreator::createNurbsSurface);
  defineNewCreationFunction<DefaultNURBSCreator>("NurbsTextureSurface",       &DefaultNURBSCreator::createNurbsTextureSurface);
  defineNewCreationFunction<DefaultNURBSCreator>("TrimmedSurface",            &DefaultNURBSCreator::createTrimmedSurface);
}

DefaultNURBSCreator::~DefaultNURBSCreator()
{
}

SFNode DefaultNURBSCreator::createContour2D() const
{
  return new Contour2D();
}

SFNode DefaultNURBSCreator::createContourPolyline2D() const
{
  return new ContourPolyline2D();
}

SFNode DefaultNURBSCreator::createNurbsCurve() const
{
  return new NurbsCurve();
}

SFNode DefaultNURBSCreator::createNurbsCurve2D() const
{
  return new NurbsCurve2D();
}

SFNode DefaultNURBSCreator::createNurbsGroup() const
{
  return new NurbsGroup();
}

SFNode DefaultNURBSCreator::createNurbsPositionInterpolator() const
{
  return new NurbsPositionInterpolator();
}

SFNode DefaultNURBSCreator::createNurbsSurface() const
{
  return new NurbsSurface();
}

SFNode DefaultNURBSCreator::createNurbsTextureSurface() const
{
  return new NurbsTextureSurface();
}

SFNode DefaultNURBSCreator::createTrimmedSurface() const
{
  return new TrimmedSurface();
}

}
}
