#include "NodeWalkingProxy.h"
#include "X3DComponentWalker.h"
#include "WalkingFunction.h"
#include "NodeVisitingProxy.h"
#include "AbstractWalker.h"
#include "Type.h"
#include "X3DNode.h"

using namespace X3DTK;
using namespace std;

NodeWalkingProxy::NodeWalkingProxy()
: X3DNodeProxy()
{
  Type::addNodeProxy(this);
  nodeVisitingProxy_ = 0;
  setComponentWalker(new AbstractWalker());
}

NodeWalkingProxy::~NodeWalkingProxy()
{
  if (autoDelete)
    for (list<X3DComponentWalker *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      (*it)->removeOneProxy();
      if ((*it)->getProxyNumber() == 0)
        delete *it;
    }    
 
  Type::removeNodeProxy(this);
}

void NodeWalkingProxy::setNodeVisitingProxy(const NodeVisitingProxy *nodeVisitingProxy)
{
  nodeVisitingProxy_ = (NodeVisitingProxy *)nodeVisitingProxy;
  for (list<X3DComponentWalker *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    (*it)->setNodeVisitingProxy(nodeVisitingProxy);
}

void NodeWalkingProxy::setComponentWalker(X3DComponentWalker *component)
{
  //replace the equivalent component.
  X3DComponentWalker *replacedComponent = 0;
  bool replaced = false;
  for (list<X3DComponentWalker *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
  {
    if ((*it)->getName() == component->getName())
    {
      replacedComponent = *it;
      *it = (X3DComponentWalker *)component; 
      replacedComponent->removeOneProxy();
      component->addOneProxy();      
      replaced = true;
    }  
  }
  
  if (replacedComponent == component)
    return;
  
  if (!replaced)
  {
    componentList_.push_back((X3DComponentWalker *)component);
    component->addOneProxy();
  }  
  
  //remove replaced component.
  if ((autoDelete) && (replacedComponent != 0) && (replacedComponent->getProxyNumber() == 0))
    delete replacedComponent;
        
  component->setNodeWalkingProxy(this);
  component->setNodeVisitingProxy(nodeVisitingProxy_);
  
  //iterating the types.
  for (unsigned int id = 0; id < walkingArray_.size(); ++id)
    walkingArray_[id] = getWalkingFunctionOf(Type::getTypeOfId(id));
}

void NodeWalkingProxy::reset()
{
  //remove components
  if (autoDelete)
    for (list<X3DComponentWalker *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      (*it)->removeOneProxy();
      if ((*it)->getProxyNumber() == 0)
        delete *it;
    }    
 
  componentList_.clear();
}

void NodeWalkingProxy::addType(const Type *type)
{
  //resize the array.
  if ((int)walkingArray_.size() <= type->getId())
    walkingArray_.resize(type->getId() + 1);

  walkingArray_[type->getId()] = getWalkingFunctionOf(type);
}

WalkingFunction *NodeWalkingProxy::getWalkingFunctionOf(const Type *type) const
{
  Type *t = (Type *)type;
  WalkingFunction *WF = 0;
  while (t != 0)
  {
    for (list<X3DComponentWalker *>::const_iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      WF = (*it)->getWalkingFunctionOf(t);
      if (WF != 0)
        break;
    }
    if (WF != 0)
      break;
    
    t = t->getParent();
  }
  
  return WF;
}

NodeWalkingProxy *X3DTK::joinNodeWalkingProxies(NodeWalkingProxy *N0, NodeWalkingProxy *N1)
{
  NodeWalkingProxy *N = new NodeWalkingProxy();
  
  for (list<X3DComponentWalker *>::const_iterator it = N0->componentList_.begin(); it != N0->componentList_.end(); ++it)
    N->setComponentWalker(*it);
  
  for (list<X3DComponentWalker *>::const_iterator it = N1->componentList_.begin(); it != N1->componentList_.end(); ++it)
    N->setComponentWalker(*it);
  
  return N;  
}
