#include "X3DTTAlgorithm.h"
#include "NodeVisitingProxy.h"
#include "NodeWalkingProxy.h"
#include "X3DNode.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

X3DTTAlgorithm::X3DTTAlgorithm()
: nvp(new NodeVisitingProxy()), nwp(new NodeWalkingProxy())
{
  nwp->setNodeVisitingProxy(nvp);
}

X3DTTAlgorithm::~X3DTTAlgorithm()
{
  delete nwp;
  delete nvp;
}

void X3DTTAlgorithm::setComponentWalker(X3DComponentWalker *component)
{
  nwp->setComponentWalker(component);
}

void X3DTTAlgorithm::setComponentVisitor(X3DComponentVisitor *component)
{
  nvp->setComponentVisitor(component);
}

void X3DTTAlgorithm::setAutoDeleteComponents(bool value)
{
  nvp->setAutoDeleteComponents(value);
  nwp->setAutoDeleteComponents(value);
}
  
void X3DTTAlgorithm::traverse(SFAbstractNode N) const
{
  if ((nwp != 0) && (N != 0))
    nwp->walk(N);
}

X3DTTAlgorithm *X3DTK::joinTTAlgorithms(X3DTTAlgorithm *A1, X3DTTAlgorithm *A2)
{
  X3DTTAlgorithm *A = new X3DTTAlgorithm();
  
  A->nvp = joinNodeVisitingProxies(A1->nvp, A2->nvp);
  A->nwp = joinNodeWalkingProxies(A1->nwp, A2->nwp);
  A->nwp->setNodeVisitingProxy(A->nvp);
  
  return A;
}

X3DTTAlgorithm *X3DTK::mergeTTAlgorithms(X3DTTAlgorithm *A1, X3DTTAlgorithm *A2)
{
  X3DTTAlgorithm *A = new X3DTTAlgorithm();
  
  delete A->nvp;
  delete A->nwp;
  
  A->nvp = joinNodeVisitingProxies(A1->nvp, A2->nvp);
  A->nwp = joinNodeWalkingProxies(A1->nwp, A2->nwp);
  A->nwp->setNodeVisitingProxy(A->nvp);
  
  delete A1;
  delete A2;
  
  return A;
}
