#include "GLAppearance.h"
#include "X3DGLMaterialNode.h"
#include "X3DGLTextureNode.h"
#include "X3DGLTextureTransformNode.h"
#include "GLLineProperties.h"
#include "GLFillProperties.h"

namespace X3DTK {
namespace GL {

Appearance::Appearance()
: X3DAppearanceNode(), _material(0), _texture(0), _textureTransform(0), _lineProperties(0), _fillProperties(0)
{
  defineTypeName("Appearance", "GLShape");
}

Appearance::Appearance(const Appearance &A)
: X3DAppearanceNode(A), _material(A._material), _texture(A._texture), _textureTransform(A._textureTransform), _lineProperties(A._lineProperties), _fillProperties(A._fillProperties)
{
}

SFNode Appearance::clone() const
{
  return new Appearance(*this);
}

Appearance::~Appearance()
{
}

void Appearance::setMaterial(const X3DMaterialNode *material) 
{
  _material = (X3DMaterialNode *)material;
}
  
void Appearance::setTexture(const X3DTextureNode *texture) 
{
  _texture = (X3DTextureNode *)texture;
}

void Appearance::setTextureTransform(const X3DTextureTransformNode *textureTransform) 
{
  _textureTransform = (X3DTextureTransformNode *)textureTransform;
}

void Appearance::setLineProperties(const LineProperties *lineProperties) 
{
  _lineProperties = (LineProperties *)lineProperties;
}

void Appearance::setFillProperties(const FillProperties *fillProperties) 
{
  _fillProperties = (FillProperties *)fillProperties;
}

bool Appearance::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    if (_material != 0)
      return false;
      
    _material = static_cast<SFNode>(N);
    N->addParent(this);
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    if (_texture != 0)
      return false;
      
    _texture = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    if (_textureTransform != 0)
      return false;
      
    _textureTransform = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    if (_lineProperties != 0)
      return false;
      
    _lineProperties = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    if (_fillProperties != 0)
      return false;
      
    _fillProperties = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  return false;
}

bool Appearance::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    _material->removeParent(this);
    _material = static_cast<SFNode>(N);
    N->addParent(this);
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    _texture->removeParent(this);
    _texture = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    _textureTransform->removeParent(this);
    _textureTransform = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    _lineProperties->removeParent(this);
    _lineProperties = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    _fillProperties->removeParent(this);
    _fillProperties = static_cast<SFNode>(N);
    N->addParent(this);
    return true;    
  }
  
  return false;
}

bool Appearance::removeChild(const SFAbstractNode &N)
{
  if (_material == N)
  {
    _material->removeParent(this);
    _material = 0;
    return true;
  }
  
  if (_texture == N)
  {
    _texture->removeParent(this);
    _texture = 0;
    return true; 
  }
  
  if (_textureTransform == N)
  {
    _textureTransform->removeParent(this);
    _textureTransform = 0;
    return true;  
  }
  
  if (_lineProperties == N)
  {
    _lineProperties->removeParent(this);
    _lineProperties = 0;
    return true;
  }
  
  if (_fillProperties == N)
  { 
    _fillProperties->removeParent(this);
    _fillProperties = 0;
    return true; 
  }
  
  return false;
}

}
}
