#include "Cone.h"

namespace X3DTK {
namespace X3D {

Cone::Cone()
: X3DGeometry3DNode(), _bottomRadius(1.0f), _height(2.0f), _side(true), _bottom(true)  
{
  defineTypeName("Cone", "Geometry3D");
}

Cone::Cone(SFFloat bottomRadius, SFFloat height, SFBool side, SFBool bottom)
: X3DGeometry3DNode(), _bottomRadius(bottomRadius), _height(height), _side(side), _bottom(bottom)
{
  defineTypeName("Cone", "Geometry3D");
}

Cone::Cone(const Cone &C)
: X3DGeometry3DNode(C), _bottomRadius(C._bottomRadius), _height(C._height), _side(C._side), _bottom(C._bottom)
{
}

SFNode Cone::clone() const
{
  return new Cone(*this);
}

Cone::~Cone()
{
}

void Cone::setBottomRadius(SFFloat bottomRadius)
{
  _bottomRadius = bottomRadius;
}

void Cone::setHeight(SFFloat height)
{
  _height = height;
}

void Cone::setSide(SFBool side)
{
  _side = side;
}

void Cone::setBottom(SFBool bottom)
{
  _bottom = bottom;
}

void Cone::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("bottomRadius");
  if (index != -1)
    _bottomRadius = element->getAttribute(index).toFloat(); 

  index = element->getIndexAttribute("height");
  if (index != -1)
    _height = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("side");
  if (index != -1)
    _side = (element->getAttribute(index).upper() == "TRUE"); 
  
  index = element->getIndexAttribute("bottom");
  if (index != -1)
    _bottom = (element->getAttribute(index).upper() == "TRUE");
}

SFString Cone::write() const
{
  SFString attr;
  if (_bottomRadius != 1.0f)
    attr += " bottomRadius=\"" + toSFString(_bottomRadius) + "\"";
  if (_height != 2.0f)  
    attr += " height=\"" + toSFString(_height) + "\"";
  if (!_side)  
    attr += " side=\"FALSE\"";
  if (!_bottom)  
    attr += " bottom=\"FALSE\"";
      
  return attr;
}

}
}
