/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            PointLight.h                              //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef POINTLIGHT
#define POINTLIGHT

#include "X3DTypes.h"
#include "X3DLightNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/lighting.html">
/// Lighting component</a> defining an PointLight
/// (<a href="../../doc/X3DSpecs/Part01/components/lighting.html#PointLight">
/// documentation</a>).

class PointLight : public X3DLightNode
{
public:
  /// Constructor.
  PointLight();
  /// Constructs a PointLight from its attributes.
  PointLight(SFFloat       ambientIntensity,
             const SFVec3f &attenuation,
             const SFColor &color,
             SFFloat       intensity,
             const SFVec3f &location,
             SFBool        on,
             SFFloat       radius);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~PointLight();

  /// Sets the attenuation attribute.
  void setAttenuation(const SFVec3f &attenuation);
  /// Sets the location attribute.
  void setLocation(const SFVec3f &location);
  /// Sets the radius attribute.
  void setRadius(SFFloat radius);

  /// Gets the attenuation attribute.
  inline const SFVec3f &getAttenuation() const {return _attenuation;};
  /// Gets the location attribute.
  inline const SFVec3f &getLocation() const {return _location;};
  /// Gets the radius attribute.
  inline SFFloat getRadius() const {return _radius;};

  /// Loads the attributes from a X3DFileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  PointLight(const PointLight &N);

private:
  /// Attenuation of the point light.
  SFVec3f _attenuation;
  /// Translation offset of the centre point of the light.
  SFVec3f _location;
  /// Maximal distance from location that may be illuminated by the light source.
  SFFloat _radius;
};

}
}

#endif
