#include "NurbsTextureSurface.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NurbsTextureSurface::NurbsTextureSurface()
  : X3DTextureCoordinateNode(),
    _controlPoint(MFVec2f()),
    _weight      (MFFloat()),
    _uDimension  (0),
    _vDimension  (0),
    _uKnot       (MFDouble()),
    _vKnot       (MFDouble()),
    _uOrder      (3),
    _vOrder      (3)
{
  defineTypeName("NurbsTextureSurface", "NURBS");
}

NurbsTextureSurface::NurbsTextureSurface(const MFVec2f  &point,
                                         const MFVec2f  &controlPoint,
                                         const MFFloat  &weight,
                                         SFInt32        uDimension,
                                         SFInt32        vDimension,
                                         const MFDouble &uKnot,
                                         const MFDouble &vKnot,
                                         SFInt32        uOrder,
                                         SFInt32        vOrder)
  : X3DTextureCoordinateNode(point),
    _controlPoint(controlPoint),
    _weight      (weight),
    _uDimension  (uDimension),
    _vDimension  (vDimension),
    _uKnot       (uKnot),
    _vKnot       (vKnot),
    _uOrder      (uOrder),
    _vOrder      (vOrder)
{
  defineTypeName("NurbsTextureSurface", "NURBS");
}

NurbsTextureSurface::NurbsTextureSurface(const NurbsTextureSurface &N)
  : X3DTextureCoordinateNode(N),
    _controlPoint(N._controlPoint),
    _weight      (N._weight),
    _uDimension  (N._uDimension),
    _vDimension  (N._vDimension),
    _uKnot       (N._uKnot),
    _vKnot       (N._vKnot),
    _uOrder      (N._uOrder),
    _vOrder      (N._vOrder)
{
}

SFNode NurbsTextureSurface::clone() const
{
  return new NurbsTextureSurface(*this);
}

NurbsTextureSurface::~NurbsTextureSurface()
{
}

void NurbsTextureSurface::setControlPoint(const MFVec2f &controlPoint)
{
  _controlPoint = controlPoint;
}

void NurbsTextureSurface::setWeight(const MFFloat &weight)
{
  _weight = weight;
}

void NurbsTextureSurface::setUDimension(SFInt32 uDimension)
{
  _uDimension = uDimension;
}

void NurbsTextureSurface::setVDimension(SFInt32 vDimension)
{
  _vDimension = vDimension;
}

void NurbsTextureSurface::setUKnot(const MFDouble &uKnot)
{
  _uKnot = uKnot;
}

void NurbsTextureSurface::setVKnot(const MFDouble &vKnot)
{
  _vKnot = vKnot;
}

void NurbsTextureSurface::setUOrder(SFInt32 uOrder)
{
  _uOrder = uOrder;
}

void NurbsTextureSurface::setVOrder(SFInt32 vOrder)
{
  _vOrder = vOrder;
}

void NurbsTextureSurface::load(const X3DFileElement *element)
{
  X3DTextureCoordinateNode::load(element);

  int index;
  index = element->getIndexAttribute("controlPoint");
  if (index != -1)
    _controlPoint = MFVec2f(element->getAttribute(index));

  index = element->getIndexAttribute("weight");
  if (index != -1)
    _weight = MFFloat(element->getAttribute(index));

  index = element->getIndexAttribute("uDimension");
  if (index != -1)
    _uDimension = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("vDimension");
  if (index != -1)
    _vDimension = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("uKnot");
  if (index != -1)
    _uKnot = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("vKnot");
  if (index != -1)
    _vKnot = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("uOrder");
  if (index != -1)
    _uOrder = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("vOrder");
  if (index != -1)
    _vOrder = element->getAttribute(index).toInt();
}

SFString NurbsTextureSurface::write() const
{
  SFString attr;
  if (!_controlPoint.empty())
    attr += " controlPoint=\"" + toSFString(_controlPoint) + "\"";
  if (!_weight.empty())
    attr += " weight=\"" + toSFString(_weight) + "\"";
  if (_uDimension != 0)
    attr += " uDimension=\"" + toSFString(_uDimension) + "\"";
  if (_vDimension != 0)
    attr += " vDimension=\"" + toSFString(_vDimension) + "\"";
  if (!_uKnot.empty())
    attr += " uKnot=\"" + toSFString(_uKnot) + "\"";
  if (!_vKnot.empty())
    attr += " vKnot=\"" + toSFString(_vKnot) + "\"";
  if (_uOrder != 3)
    attr += " uOrder=\"" + toSFString(_uOrder) + "\"";
  if (_vOrder != 3)
    attr += " vOrder=\"" + toSFString(_vOrder) + "\"";
      
  return X3DTextureCoordinateNode::write() + attr;
}

}
}
