/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TrimmedSurface.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIMMEDSURFACE_H
#define TRIMMEDSURFACE_H

#include "X3DTypes.h"
#include "X3DParametricGeometryNode.h"

namespace X3DTK {
namespace X3D {

class Contour2D;
class NurbsSurface;

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/NURBS.html">
/// NURBS component</a> defining a TrimmedSurface
/// (<a href="../../doc/X3DSpecs/Part01/components/NURBS.html#TrimmedSurface">
/// documentation</a>).

class TrimmedSurface : public X3DParametricGeometryNode
{
public:
  /// Constructor.
  TrimmedSurface();
  /// Constructs an TrimmedSurface from its attributes.
  TrimmedSurface(const NurbsSurface *surface);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~TrimmedSurface();
  
  /// Sets the Surface attribute.
  void setSurface(const NurbsSurface *surface);

  /// Gets the TrimmingContour attribute.
  inline MFNode getTrimmingContour() const {return _trimmingContour;};
  /// Gets the Surface attribute.
  inline SFNode getSurface() const {return _surface;};

  /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);

protected:
  /// Copy constructor.       
  TrimmedSurface(const TrimmedSurface &N);

private:
  /// Set of Contour2D nodes.
  MFNode _trimmingContour;
  /// NurbsSurface node to be trimmed.
  SFNode _surface;
};

}
}

#endif
