#include "TriangleStripSet.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TriangleStripSet::TriangleStripSet()
: X3DComposedGeometryNode(), _convex(true), _creaseAngle(0.0f)
{
  defineTypeName("TriangleStripSet", "Rendering");
}

TriangleStripSet::TriangleStripSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool convex, SFFloat creaseAngle, SFBool normalPerVertex, SFBool solid, const MFInt32 &stripCount)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
_convex(convex), _creaseAngle(creaseAngle), _stripCount(stripCount)
{
  defineTypeName("TriangleStripSet", "Rendering");
}

TriangleStripSet::TriangleStripSet(const TriangleStripSet &T)
: X3DComposedGeometryNode(T), _convex(T._convex), _creaseAngle(T._creaseAngle), _stripCount(T._stripCount)
{
}

SFNode TriangleStripSet::clone() const
{
  return new TriangleStripSet(*this);
}

TriangleStripSet::~TriangleStripSet()
{
}

void TriangleStripSet::setConvex(SFBool convex)
{
  _convex = convex;
}

void TriangleStripSet::setCreaseAngle(SFFloat creaseAngle)
{
  _creaseAngle = creaseAngle;
}

void TriangleStripSet::setStripCount(const MFInt32 &stripCount)
{
  _stripCount  = stripCount;
}

void TriangleStripSet::load(const X3DFileElement *element)
{
  X3DComposedGeometryNode::load(element);
  
  int index;
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    _convex = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    _creaseAngle = element->getAttribute(index).toFloat(); 
    
  index = element->getIndexAttribute("stripCount");  
  if (index != -1)
    _stripCount = MFInt32(element->getAttribute(index));  
}

SFString TriangleStripSet::write() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::write();
  
  if (!_convex)  
    attr += " convex=\"FALSE\"";
  if (_creaseAngle != 0.0f)
    attr += " creaseAngle=\"" + toSFString(_creaseAngle) + "\"";
  if (!_stripCount.empty())
    attr += " stripCount=\"" + toSFString(_stripCount) + "\"";
    
  return attr;
}

}
}
