#include "X3DComposedGeometryNode.h"
#include "X3DColorNode.h"
#include "X3DCoordinateNode.h"
#include "X3DNormalNode.h"
#include "X3DTextureCoordinateNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

X3DComposedGeometryNode::X3DComposedGeometryNode()
: X3DGeometry3DNode(), _color(0), _coord(0), _normal(0), _texCoord(0), _ccw(true), _colorPerVertex(true), _normalPerVertex(true), _solid(true)
{
  defineTypeName("X3DComposedGeometryNode");
}

X3DComposedGeometryNode::X3DComposedGeometryNode(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool normalPerVertex, SFBool solid)
: X3DGeometry3DNode(), _color((SFNode)color), _coord((SFNode)coord), _normal((SFNode)normal), _texCoord((SFNode)texCoord), _ccw(ccw), _colorPerVertex(colorPerVertex), _normalPerVertex(normalPerVertex), _solid(solid)
{
  defineTypeName("X3DComposedGeometryNode");
}

           
X3DComposedGeometryNode::X3DComposedGeometryNode(const X3DComposedGeometryNode &N)
: X3DGeometry3DNode(N), _color(N._color), _coord(N._coord), _normal(N._normal), _texCoord(N._texCoord), _ccw(N._ccw), _colorPerVertex(N._colorPerVertex), _normalPerVertex(N._normalPerVertex), _solid(N._solid)
{
}

SFNode X3DComposedGeometryNode::clone() const
{
  return 0;
}

X3DComposedGeometryNode::~X3DComposedGeometryNode()
{
}

void X3DComposedGeometryNode::setColor(const X3DColorNode *color)
{
  removeLink(this, _color);
  _color = (SFNode)color;
  addLink(this, _color);
}

void X3DComposedGeometryNode::setCoord(const X3DCoordinateNode *coord)
{
  removeLink(this, _coord);
  _coord = (SFNode)coord;
  addLink(this, _coord);
}

void X3DComposedGeometryNode::setNormal(const X3DNormalNode *normal)
{
  removeLink(this, _normal);
  _normal = (SFNode)normal;
  addLink(this, _normal);
}

void X3DComposedGeometryNode::setTexCoord(const X3DTextureCoordinateNode *texCoord)
{
  removeLink(this, _texCoord);
  _texCoord = (SFNode)texCoord;
  addLink(this, _texCoord);
}

void X3DComposedGeometryNode::setCcw(SFBool ccw)
{
  _ccw = ccw;
}

void X3DComposedGeometryNode::setColorPerVertex(SFBool colorPerVertex)
{
  _colorPerVertex = colorPerVertex;
}

void X3DComposedGeometryNode::setNormalPerVertex(SFBool normalPerVertex)
{
  _normalPerVertex = normalPerVertex;
}

void X3DComposedGeometryNode::setSolid(SFBool solid)
{
  _solid = solid;
}

bool X3DComposedGeometryNode::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    if (_color != 0)
      return false;
    
    setColor(static_cast<X3DColorNode *>(N)); 
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    if (_coord != 0)
      return false;
      
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DNormalNode *>(N) != 0)
  {
    if (_normal != 0)
      return false;
      
    setNormal(static_cast<X3DNormalNode *>(N));   
    return true;
  }
  
  if (dynamic_cast<X3DTextureCoordinateNode *>(N) != 0)
  {
    if (_texCoord != 0)
      return false;
    
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));   
    return true;    
  }
  return false;
}

bool X3DComposedGeometryNode::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    setColor(static_cast<X3DColorNode *>(N)); 
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DNormalNode *>(N) != 0)
  {
    setNormal(static_cast<X3DNormalNode *>(N));  
    return true;
  }
  
  if (dynamic_cast<X3DTextureCoordinateNode *>(N) != 0)
  {
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "X3DComposedGeometryNode::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

bool X3DComposedGeometryNode::removeChild(const SFAbstractNode &N)
{
  if (_color == N)
  {
    setColor(0);
    return true;
  } 
   
  if (_coord == N)
  {
    setCoord(0);
    return true;
  } 

  if (_normal == N)
  {
    setNormal(0);
    return true;
  } 
   
  if (_texCoord == N)
  {
    setTexCoord(0);
    return true;
  } 
  
  cerr << "X3DComposedGeometryNode::removeChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

void X3DComposedGeometryNode::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("ccw");
  if (index != -1)
    _ccw = (element->getAttribute(index).upper() == "TRUE");
  
  index = element->getIndexAttribute("colorPerVertex");  
  if (index != -1)
    _colorPerVertex = (element->getAttribute(index).upper() == "TRUE");  

  index = element->getIndexAttribute("normalPerVertex");
  if (index != -1)
    _normalPerVertex = (element->getAttribute(index).upper() == "TRUE");  
  
  index = element->getIndexAttribute("solid");  
  if (index != -1)
    _solid = (element->getAttribute(index).upper() == "TRUE");
}

SFString X3DComposedGeometryNode::write() const
{
  SFString attr;
  if (!_ccw)
    attr += " ccw=\"FALSE\"";
  if (!_colorPerVertex)  
    attr += " colorPerVertex=\"FALSE\"";
  if (!_normalPerVertex)
    attr += " normalPerVertex=\"FALSE\"";
  if (!_solid)  
    attr += " solid=\"FALSE\"";  
  
  return attr;
}

}
}
