#include "GLBuilder.h"
#include "DefaultDFS.h"
#include "GLBuilderCoreVisitor.h"
#include "GLBuilderGeometry3DVisitor.h"
#include "GLBuilderGroupingVisitor.h"
#include "GLBuilderRenderingVisitor.h"
#include "GLBuilderShapeVisitor.h"
#include "GLBuilderTexturingVisitor.h"
#include "GLScene.h"

namespace X3DTK {
namespace X3D {

GLBuilder::GLBuilder()
: X3DGLBuilder()
{
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
  dfs = new DefaultDFS();
  
  dfs->setComponentVisitor(new GLBuilderCoreVisitor());
  dfs->setComponentVisitor(new GLBuilderGeometry3DVisitor());
  dfs->setComponentVisitor(new GLBuilderGroupingVisitor());
  dfs->setComponentVisitor(new GLBuilderRenderingVisitor());
  dfs->setComponentVisitor(new GLBuilderShapeVisitor());
  dfs->setComponentVisitor(new GLBuilderTexturingVisitor());
}

GLBuilder::~GLBuilder()
{
  delete dfs;
}

void GLBuilder::setComponentVisitor(X3DComponentVisitor *component)
{
  dfs->setComponentVisitor(component);
}

GL::SFNode GLBuilder::build(SFNode N)
{
  globalVariables->init();
  dfs->traverse(N);
  globalVariables->finish();
  
  return globalVariables->getRoot();
}

}
}
