#include "GLBuilderRenderingVisitor.h"
#include "IndexedLineSet.h"
#include "PointSet.h"
#include "TriangleFanSet.h"
#include "TriangleSet.h"
#include "TriangleStripSet.h"
#include "GLIndexedLineSet.h"
#include "GLPointSet.h"
#include "GLTriangleFanSet.h"
#include "GLTriangleSet.h"
#include "GLTriangleStripSet.h"
#include "X3DGeometryNode.h"

namespace X3DTK {
namespace X3D {

GLBuilderRenderingVisitor::GLBuilderRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLBuilderRenderingVisitor, IndexedLineSet>(&GLBuilderRenderingVisitor::enterIndexedLineSet);
  defineNewEnterFunction<GLBuilderRenderingVisitor, PointSet>(&GLBuilderRenderingVisitor::enterPointSet);
  defineNewEnterFunction<GLBuilderRenderingVisitor, TriangleFanSet>(&GLBuilderRenderingVisitor::enterTriangleFanSet);
  defineNewEnterFunction<GLBuilderRenderingVisitor, TriangleSet>(&GLBuilderRenderingVisitor::enterTriangleSet);
  defineNewEnterFunction<GLBuilderRenderingVisitor, TriangleStripSet>(&GLBuilderRenderingVisitor::enterTriangleStripSet);

  // WalkOn functions
  defineNewWalkOnFunction<GLBuilderRenderingVisitor, X3DGeometryNode>(&GLBuilderRenderingVisitor::walkOnX3DGeometryNode);

  
  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderRenderingVisitor::~GLBuilderRenderingVisitor()
{
}

void GLBuilderRenderingVisitor::enterIndexedLineSet(IndexedLineSet *I) const
{
  GL::IndexedLineSet *GI = new GL::IndexedLineSet();
  GI->setX3DReference(I);
  globalVariables->getTop()->addChild(GI);
  globalVariables->pushNode(GI);
}

void GLBuilderRenderingVisitor::enterPointSet(PointSet *P) const
{
  GL::PointSet *GP = new GL::PointSet();
  GP->setX3DReference(P);
  globalVariables->getTop()->addChild(GP);
  globalVariables->pushNode(GP);
}

void GLBuilderRenderingVisitor::enterTriangleFanSet(TriangleFanSet *T) const
{
  GL::TriangleFanSet *GT = new GL::TriangleFanSet();
  GT->setX3DReference(T);
  globalVariables->getTop()->addChild(GT);
  globalVariables->pushNode(GT);
}

void GLBuilderRenderingVisitor::enterTriangleSet(TriangleSet *T) const
{
  GL::TriangleSet *GT = new GL::TriangleSet();
  GT->setX3DReference(T);
  globalVariables->getTop()->addChild(GT);
  globalVariables->pushNode(GT);
}

void GLBuilderRenderingVisitor::enterTriangleStripSet(TriangleStripSet *T) const
{
  GL::TriangleStripSet *GT = new GL::TriangleStripSet();
  GT->setX3DReference(T);
  globalVariables->getTop()->addChild(GT);
  globalVariables->pushNode(GT);
}

bool GLBuilderRenderingVisitor::walkOnX3DGeometryNode(X3DGeometryNode *N, SFAbstractNode child) const
{
  return false;
}

}
}
