#include "SceneSaver.h"
#include "DefaultDFS.h"
#include "SceneSaverCoreVisitor.h"

#include <fstream>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaver::SceneSaver(DFS *dfs)
: X3DSceneSaver()
{
  globalVariables = GVManager::getInstanceOf<SceneSaverGlobalVariables>();
  
  if (dfs == 0)
  {
    this->dfs = new DefaultDFS();
    defaultDFSUSE = true;
  }  
  else
  {
    this->dfs = dfs;
    defaultDFSUSE = false;
  }
    
  this->dfs->setComponentVisitor(new SceneSaverCoreVisitor());
}

SceneSaver::~SceneSaver()
{
  if (defaultDFSUSE)
    delete dfs;
}

void SceneSaver::setComponentVisitor(X3DComponentVisitor *component)
{
  dfs->setComponentVisitor(component);
}

void SceneSaver::saveAs(Scene *S, const SFString &fileName) const
{
  if (S == 0)
  {
    cerr << "unable to write to file, because the Scene is NULL!" << endl;
    return;
  }
    
  ofstream output(fileName);
  
  output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
  output << "<X3D>" << endl;
  
  dfs->traverse(S);
  MFString text = globalVariables->getOutput();
  
  for (MFString::const_iterator it = text.begin(); it != text.end(); ++it)
    output << (*it) << endl;

  output << "</X3D>" << endl;
}

void SceneSaver::save(Scene *S) const
{
  saveAs(S, S->getFileName());
}

}
}
