#include "SceneSaverGlobalVariables.h"
#include "X3DAbstractNode.h"
#include "X3DNode.h"

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaverGlobalVariables::SceneSaverGlobalVariables()
: GlobalVariables(), _lineNumber(-1), _tab("  ")
{
}

SceneSaverGlobalVariables::~SceneSaverGlobalVariables()
{
}

void SceneSaverGlobalVariables::pushHasUSE(bool value)
{
  _hasUSEStack.push_front(value);
}

void SceneSaverGlobalVariables::newLine()
{
  ++_lineNumber;
  SFString l;
  _output.push_back(SFString(_tab));
}

void SceneSaverGlobalVariables::pushCurrentLine()
{
  _lineStack.push_front(_lineNumber);
}

void SceneSaverGlobalVariables::writeCurrentLine(const SFString &string)
{
  _output.back() += string;
}

void SceneSaverGlobalVariables::writeOpeningLine(const SFString &string)
{
  _output[_lineStack.front()] += string;
}

void SceneSaverGlobalVariables::addTab()
{
  _tab += "  ";
}

void SceneSaverGlobalVariables::removeTab()
{
  _tab.erase(0, 2);
}

bool SceneSaverGlobalVariables::hasChild()
{
  return (_lineNumber == _lineStack.front());
}

void SceneSaverGlobalVariables::popHasUSE()
{
  _hasUSEStack.pop_front();
}

void SceneSaverGlobalVariables::popLine()
{
  _lineStack.pop_front();
}

SFString SceneSaverGlobalVariables::getDEF(const SFNode &N)
{
  //finding name
  SFString DEF = X3DAbstractNode::DEFDict.getNameOfNode(N);
  if (DEF == "")
    return "";
  
  //the DEF doesn't exist
  //inserting the new DEF
  bool found = false;
  for (list<SFString>::const_iterator it = _DEFList.begin(); it != _DEFList.end(); ++it)
    if (*it == DEF)
    {
      found = true;
      break;
    }  
    
  if (!found)
  {
    _DEFList.push_back(DEF);
    return SFString(" DEF=\"") + DEF + "\"";  
  }
  return ""; 
}

SFString SceneSaverGlobalVariables::getUSE(const SFNode &N) const
{
  //finding name
  SFString DEF = X3DAbstractNode::DEFDict.getNameOfNode(N);
  if (DEF == "")
    return "";
  
  bool found = false;
  for (list<SFString>::const_iterator it = _DEFList.begin(); it != _DEFList.end(); ++it)
    if (*it == DEF)
    {
      found = true;
      break;
    }  
    
  if (found)
    return SFString(" USE=\"") + DEF + "\"";

  return "";  
}

}
}
